# Generated by Django 4.2.13 on 2024-07-10 18:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0216_enterprisegroupmembership_errored_at_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='disable_expiry_messaging_for_learner_credit',
            field=models.BooleanField(default=False, help_text='If checked, learners and admins will not receive notifications leading up to the expiration date for learner credit plans. This includes removing the expiration date from the subsidy box for upcoming expirations. The subsidy summary box will not display for expired plans. Other notifications signaling the expiration (and loss of usability) itself will still appear.', verbose_name='Disable expiration messaging for learner credit'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='disable_expiry_messaging_for_learner_credit',
            field=models.BooleanField(default=False, help_text='If checked, learners and admins will not receive notifications leading up to the expiration date for learner credit plans. This includes removing the expiration date from the subsidy box for upcoming expirations. The subsidy summary box will not display for expired plans. Other notifications signaling the expiration (and loss of usability) itself will still appear.', verbose_name='Disable expiration messaging for learner credit'),
        ),
    ]
