# Generated by Django 3.2.18 on 2023-05-03 14:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0170_auto_20230301_1627'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomer',
            name='auth_org_id',
            field=models.CharField(blank=True, help_text='Enterprise Customer auth organization id', max_length=80, null=True),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomer',
            name='auth_org_id',
            field=models.CharField(blank=True, help_text='Enterprise Customer auth organization id', max_length=80, null=True),
        ),
    ]
