# Generated by Django 4.2.20 on 2025-04-22 21:45

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields
import uuid


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('enterprise', '0232_backfill_ecu_table_user_fk_data_migration'),
    ]

    operations = [
        migrations.CreateModel(
            name='OnboardingFlow',
            fields=[
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('title', models.CharField(help_text='Title summarizing the onboarding flow.', max_length=255)),
                ('active', models.BooleanField(default=False, help_text='Specifies whether this flow is active and shown to admin users in the onboarding tour.')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
