# Generated by Django 3.2.12 on 2022-03-24 15:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('integrated_channel', '0006_contentmetadataitemtransmission_deleted_at'),
    ]

    operations = [
        migrations.DeleteModel(
            name='LearnerDataTransmissionAudit',
        ),
    ]
