# Generated by Django 4.2.19 on 2025-04-28 16:42

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('enterprise', '0233_onboardingflow'),
    ]

    operations = [
        migrations.SeparateDatabaseAndState(
            database_operations = [],
            state_operations = [
                migrations.AlterField(
                    model_name='enterprisecustomeruser',
                    name='user_fk',
                    field=models.ForeignKey(db_column='user_fk', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='enterprise_customer_users', to=settings.AUTH_USER_MODEL),
                ),
                migrations.AlterField(
                    model_name='historicalenterprisecustomeruser',
                    name='user_fk',
                    field=models.ForeignKey(blank=True, db_column='user_fk', db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to=settings.AUTH_USER_MODEL),
                ),
                migrations.AddIndex(
                    model_name='enterprisecustomeruser',
                    index=models.Index(fields=['user_fk'], name='idx_enterprise_user_fk'),
                ),
            ]
        )
    ]
